
> module BestFirstSearch where
> import List

****************************
* Best First Search 
****************************

A general-purpose impl. of RBFS.

> type Cost = Integer

The class State takes two types -- a problem and a state.

> class SearchProblem s where -- aka Path
>  cost       :: s -> s -> Cost -- g(x)
>  estimatedRemainingCost :: s -> s -> Cost -- h(x), should be underestimate (admissable)
>  isSolution :: s -> s -> Bool
>  expand     :: s -> s -> [s]

> bestFirstSearch :: (SearchProblem s) => s -> s -> Cost -> Maybe s
> bestFirstSearch p s limit = case newSol of
>                             (Just _) -> newSol
>		              Nothing  -> case sols of
>                                         [] -> Nothing
>                                         n  -> if (cost p minSols) < limit
>                                               then Just minSols 
>                                               else Nothing
>   where ns     = expand p s
>         sols   = filter (\s -> isSolution p s) ns
>         rest   = filter (\s -> not (isSolution p s)) ns
>         l      = minimum (limit:(map (cost p) sols))
>         newSol = bfs p rest l Nothing
>         minSols = minimumBy f sols
>         f s s' = compare (cost p s) (cost p s')

> bfs :: (SearchProblem s) => s -> [s] -> Cost -> Maybe s -> Maybe s
> bfs _ []     _     sol = sol
> bfs p (s:ss) limit sol = bfs p ss l sol'
>   where sol' = if ((f s) < limit) then s'' else sol
>         s'   = bestFirstSearch p s limit
>         s''  = case sol of
>                Nothing  -> s'
>                (Just x) -> case s' of 
>                            Nothing  -> sol
>                            (Just y) -> if ((g y) < (g x)) 
>                                        then s' else sol
>         l    = case sol' of 
>                Nothing  -> limit 
>                (Just x) -> g x
>         f x  = (g x) + (h x)
>         g    = cost p
>         h    = estimatedRemainingCost p

RBFS Test 

> type SimpleMapEntry = ((String, String), Integer)
> type SimpleMap = [SimpleMapEntry]

> sampleMap :: SimpleMap
> sampleMap = [((a, c), 2), ((a, b), 3),
>              ((b, d), 7), 
>              ((d, e), 2), ((d, f), 3),
>              ((e, f), 6)]
>  where a = "A"
>        b = "B"
>        c = "C"
>        d = "D"
>        e = "E"
>        f = "F"

> instance SearchProblem SimpleMap where
>   cost _ [] = 0
>   cost _ ns = sum (map snd ns)
>   estimatedRemainingCost p s = toInteger (((length p) - (length s)) * minimumCost)
>     where minimumCost = 1
>   isSolution p [] = False 
>   isSolution p ns@(n:_) = ((getDest (last ns)) == "F") && ((getName n) == "A")
>   --expand :: SimpleMap -> SimpleMap -> [SimpleMap]
>   expand p [] = map (\n -> [n]) (filter (\((n, d), c) -> n == "A") p)
>   expand p s = map (\x -> s ++ [x]) (filter f p)
>     where n = last s
>           f :: SimpleMapEntry -> Bool 
>           f x = (getName x) == (getDest n)

> testBfs = bestFirstSearch sampleMap [] 1000 -- correcct solution is ABDF

> getName :: SimpleMapEntry -> String
> getName n = fst (fst n) 

> getDest x = snd (fst x)

***************
* Old Code
***************

This doesn't work and I'm not sure why...

< class SearchProblem p s where -- aka Path
<  cost       :: p -> s -> Cost -- g(x)
<  estimatedRemainingCost :: p -> s -> Cost -- h(x), should be underestimate (admissable)
<  isSolution :: p -> s -> Bool
<  expand     :: p -> s -> [s]

for sax impl...
< expand p s = if (isSolution p s) then [] else x
<   where x = nyi -- impl-specific.  extend s repeatedly until you reach the 1st branch, then return
                                     all states there.  If reach solution then return [solution]

