\subsection{An Example of Etude Generation}

Given a very simple etude (shown in figure 2), the initial (empty) performer database, and a set of performance evaluations, the etude generation functions can be used to create the etude shown in Figure 3.  

\begin{verbatim}

> module EtudeGenerationExample where
> import Ratio
> import Basics
> import ComposersWorkbench
> import Scores
> import Performer
> import PerformanceEvaluation
> import EtudeGeneration

> simpleEtude :: Score
> simpleEtude = Score "A very simple etude" (4%4) 108 (Key C Major) m
>   where m  = sequencePhrases [p1, p2]
>         p1 = makeMelody oct p1PCS p1Rhythm
>         p2 = makeMelody oct p2PCS p2Rhythm
>         p1PCS = [4,2,0,2,4,0,0,4]
>         p2PCS = [5,4,2,4,5,2,0]
>         p1Rhythm = en4qn2 :&: hn :&: hn
>         p2Rhythm = en4qn2 :&: (NoteR 1)
>         en4qn2 = en :&: en :&: en :&: en :&: qn :&: qn
>         qn            = NoteR (1 % 4)
>         en            = NoteR (1 % 8)
>         hn            = NoteR (1 % 2)
>         oct = (12*5)

> rttEtude2 :: Score
> rttEtude2 = generateEtude tm ts title
>   where title = "Round Trip Test: Etude Based on the Simple Etude"
>         ts :: [FingeringTrans] 
>         ts = transitionsToTest tm (phraseNum 1 simpleEtude) bpm 
>                                (timeSigToBPMeas timeS)
>         (Score t timeS bpm _ _) = simpleEtude 
>         tm :: TechniqueMap -- updated Technique Map
>         tm = updateWithEvals [simpleEtude] pes tm0
>         pes :: [PerformanceEvaluation]
>         pes = [CanPlay (PhraseNum 0 t), 
>                CannotPlay (PhraseNum 1 t) TrickyFingering]
>         (Performer _ (Abilities tm0)) = defPerformer

> defPerformer = makePerformer "Sample Performer with Default values"

\end{verbatim}

\begin{figure}[h]
\epsfig{figure=images/figure2.eps, width=12cm}
\caption{A simple etude.}
\end{figure}

\begin{figure}[h]
\epsfig{figure=images/figure3.eps, width=12cm}
\caption{An etude generated automatically, based on the second phrase of the etude shown in Figure 2.}
\end{figure}

