
\subsection{A Typsetting Example}

  The {\tt ScoresExample} module includes a short score that demonstrates the typesetting subsystem.  Data types from Haskore and the {\tt ComposersWorkbench} module are used to encode the first three phrases from the Finale from Theme and variations, found on page 242 of Paul DeVille's Universal Method for the Saxophone. \cite{umfts-b}

\begin{verbatim}

> module ScoresExample where 
> import Ratio
> import Basics
> import ComposersWorkbench
> import Scores

> umTandV :: Score
> umTandV = Score title (4%4) 108 (Key D Major) m
>   where title         = "Finale from Theme and variations, " ++ 
>                         "Universal Method, p.242"
>         m             = (mp phrase1) :+: (mp phrase1) :+: (mp phrase3)
>         phrase1       = makeMelody oct phrase1PCS phrase1Rhythm
>         phrase3       = makeMelody oct phrase3PCS phrase3Rhythm
>         phrase1PCS    = [0, 4, -1, 0, -5, 4, -1, 0, -5, 7, 5, 4, 2, 0, 
>                          -1, -3, -5, -7, -8, -10, -12, -13, -10, -3, -5]
>         phrase1Rhythm = stn3 :&: stn :&: qnPStn :&: stn3 :&: qn :&: 
>                         stn8 :&: stn8
>         phrase3PCS    = [-8, -12, -10, -8, -7, -5, -3, -2, 0, -2, -3, 
>                          -2, 2, 0, -2] ++ 
>                         [-3, -4, -3, 1, 2, 0, -1, -2, -1, 3, 4, 2] ++ 
>                         [0, -1, 0, 4, 5, 4, 2, 4, 2, 1, 2, 6, 7, 5]
>         phrase3Rhythm = phrase31R :&: motive32R :&: motive32R :&: 
>                         motive32R :&: stn8
>         phrase31R     = en :&: stn :&: stn :&: stn4 :&: stn8
>         motive32R     = den :&: stn :&: stn4
>         stn3          = stn :&: stn :&: stn
>         stn4          = stn3 :&: stn
>         stn8          = stn4 :&: stn4
>         stn           = NoteR (1 % 16)
>         qnPStn        = NoteR (5 % 16)
>         qn            = NoteR (1 % 4)
>         en            = NoteR (1 % 8)
>         den           = NoteR (3 % 16)
>         oct           = (12*5)+2
>         mp p          = (Phrase [] p)

> renderUMTandV = renderLPScore (score2LPScore umTandV)

\end{verbatim}

The value of {\tt renderUMTandV} is the following text, which LilyPond interprets as shown in Figure 1.

\begin{verbatim}
%{Generated from a Score by Scores.lhs.%}
\header{title = "Finale from Theme and variations, Universal Method, p.242"}
{\key d \major \tempo 4=108 d''16 fis''16 cis''16 d''16 a'4 ~ a'16 fis''16 
cis''16 d''16 a'4 a''16 g''16 fis''16 e''16 d''16 cis''16 b'16 a'16 g'16 
fis'16 e'16 d'16 cis'16 e'16 b'16 a'16 d''16 fis''16 cis''16 d''16 a'4 ~ 
a'16 fis''16 cis''16 d''16 a'4 a''16 g''16 fis''16 e''16 d''16 cis''16 b'16 
a'16 g'16 fis'16 e'16 d'16 cis'16 e'16 b'16 a'16 fis'8 d'16 e'16 fis'16 
g'16 a'16 b'16 c''16 d''16 c''16 b'16 c''16 e''16 d''16 c''16 b'8. ais'16 
b'16 dis''16 e''16 d''16 cis''8. c''16 cis''16 f''16 fis''16 e''16 d''8. 
cis''16 d''16 fis''16 g''16 fis''16 e''16 fis''16 e''16 dis''16 e''16 
gis''16 a''16 g''16 \bar "|." }
\version "2.6.0"
\end{verbatim}

\begin{figure}[h]
\epsfig{figure=images/figure1.eps, width=12cm}
\caption{The final result of typesetting the example score.}
\end{figure}


